/* map.js */

"use strict";

// load images
var imgW = document.createElement('img'); imgW.src = 'images/mars1.png';
var imgE = document.createElement('img'); imgE.src = 'images/mars2.png';

/* --------------------------------------------------------------
   addGrid (ctx, widthPx, heightPx)
   --------------------------------------------------------------*/
function addGrid(ctx, width,height) {
    ctx.globalAlpha=0.8;
    ctx.lineWidth=1.0;
    for(var lat=60; lat>=-60; lat-=30) {
        ctx.strokeStyle=(lat==0)? "#e00000": "#a06040";
        ctx.beginPath(); 
        ctx.moveTo(0, (90-lat)/180*height);
        ctx.lineTo(width-1, (90-lat)/180*height);
        ctx.stroke();
    }
    for(var lon=-150; lon<=150; lon+=30) {
        ctx.strokeStyle=(lon==0)? "#e00000": "#a08060";
        ctx.beginPath(); 
        ctx.moveTo((lon+180)/360*width, 0);
        ctx.lineTo((lon+180)/360*width, height-1);
        ctx.stroke();
    }

    ctx.globalAlpha=1.0;
    ctx.font="12px Arial"; ctx.fillStyle="#ac8d6c";
    ctx.textAlign = 'left';
    for(var lat=60; lat>=-60; lat-=30) 
        ctx.fillText(lat, 5,(90-lat)/180*height);
    ctx.textAlign = 'center';
    for(var lon=-150; lon<=150; lon+=30) 
        ctx.fillText(lon, (lon+180)/360*width, 12);
} // addGrid

/* --------------------------------------------------------------
   addApts (ctx, widthPx, heightPx)
   --------------------------------------------------------------*/
function addApts(ctx, width,height) {
    ctx.globalAlpha = 1;
    ctx.font="10px Arial";
    ctx.textAlign = 'center';
    for(var label in apts) {
        var x = (apts[label].loc.p.lon+180)/360*width;
        var y = (90-apts[label].loc.p.lat)/180*height;
        ctx.fillStyle = '#000000';
        ctx.strokeStyle = '#ffcc99'; ctx.lineWidth = 1;
        ctx.beginPath(); ctx.arc(x, y, 3, 0,2*Math.PI); ctx.fill(); ctx.stroke();
        ctx.fillStyle = '#ffdd66';
        ctx.fillText(label, x, y-5);
    }
} // addApts

/* --------------------------------------------------------------
   addRoute (ctx, widthPx, heightPx, icao1, icao2)
   --------------------------------------------------------------*/
function addRoute(ctx, width, height, icao1, icao2) {

    function refine(n1) {
      var n2 = n1+1;
      // Distance (in degrees) in Mercator projection
      var posX1 = route[n1].lon;
      var posX2 = route[n2].lon;
      var disX = (posX2-posX1 + 360 + 180)%360 - 180;
      var disY = route[n2].lat - route[n1].lat;
      var dis = Math.sqrt(disX*disX + disY*disY);
      // If a leg is longer than 10 degrees, then subdivide it
      if(dis > 10.0) {
          var mid = route[n1].midpointTo(route[n1+1]);
          // Insert 'mid' straight after n1
          for(var i=route.length-1; i>n1; i--) route[i+1] = route[i];
          route[n1+1] = mid;
          n2 = refine(n1);
          n2 = refine(n2);
      }
      return n2;
    } // refine

    function dateline() {
      for(var n1=0; n1<route.length-1; n1++) {
          // Does route cross the dateline?
          if((route[n1].lon>170 && route[n1+1].lon<-170) || 
             (route[n1].lon<-170 && route[n1+1].lon>170)) {
              //var brng1 = route[n1].bearingTo(route[n1+1]);
              //var p2 = new LatLon(0,180);
              //var brng2 = 0;
              //var latCrossing = LatLon.intersection(route[n1],brng1,p2,brng2).lat;
              var latCrossing = (route[n1].lat + route[n1+1].lat)/2;  // ugly approx!
              for(var i=route.length-1; i>n1; i--) route[i+2] = route[i];
              if(route[n1].lon>170 && route[n1+1].lon<-170) {
                  // Crossing from E to W
                  route[n1+1] = new LatLon(latCrossing,180);
                  route[n1+2] = new LatLon(latCrossing,-180);
              } else if(route[n1].lon<-170 && route[n1+1].lon>170) {
                  // Crossing from W to E
                  route[n1+1] = new LatLon(latCrossing,-180);
                  route[n1+2] = new LatLon(latCrossing,180);
              }
              break;
          }
      }
    } // dateline

    // Compute GC route
    var route = [apts[icao1].loc.p, apts[icao2].loc.p];
    refine(0);
    dateline();

    // Plot GC route
    ctx.strokeStyle = '#5555ff'; ctx.lineWidth = 3;
    var drawing = false;
    for(var i=0; i<route.length; i++) {
        if(!drawing) {
           var x1 = (route[i].lon+180)/360*width;
           var y1 = (90-route[i].lat)/180*height;
           ctx.beginPath();
           ctx.moveTo(x1,y1);
           drawing = true;
           continue;
        }
        var x2 = (route[i].lon+180)/360*width;
        var y2 = (90-route[i].lat)/180*height;
        ctx.lineTo(x2,y2);
        if((route[i].lon==180 && route[i+1].lon==-180) || 
           (route[i].lon==-180 && route[i+1].lon==180)) {
           ctx.stroke();
           drawing = false;
           continue;
        }
    }
    ctx.stroke();
    //console.log("Route comprises "+route.length+" points");    
} // addRoute

/* --------------------------------------------------------------
   redrawMap (icao1, icao2)
   --------------------------------------------------------------*/
function redrawMap(icao1, icao2) {
    // draw images
    var canvas = document.querySelector('canvas#map');
    var ctx = canvas.getContext('2d');
    ctx.drawImage(imgW, 0,0,              canvas.width/2,canvas.height);
    ctx.drawImage(imgE, canvas.width/2,0, canvas.width/2,canvas.height);
    // add rest
    addGrid(ctx, canvas.width, canvas.height);
    addRoute(ctx,canvas.width, canvas.height, icao1, icao2);
    addApts(ctx, canvas.width, canvas.height);

} // insertMapImages

/* --------------------------------------------------------------
   createClickHandler ()
   --------------------------------------------------------------*/
function createClickHandler() {
    var canvas = document.querySelector('canvas#map');
    canvas.onclick = function(event) {
        // See Flanagan p 395 for more about position
        var lon = 360.0*(event.clientX+window.pageXOffset-canvas.offsetLeft)/canvas.width - 180.0;
        var lat = 90.0 - 180.0*(event.clientY+window.pageYOffset-canvas.offsetTop)/canvas.height;
        var tol = 2.0;   // tolerance when clicking on an apt; in degrees
        //debug("Click at "+lat+","+lon);
        for(var label in apts) {
            var dLon = lon - apts[label].loc.p.lon;
            var dLat = lat - apts[label].loc.p.lat;
            if(Math.abs(dLon)<tol && Math.abs(dLat)<tol) {
                handleMapClick(document.getElementById("calcApt2").innerHTML,label);

                // Show information for this airport
                //debug("Click at "+label);
                var aptTables = document.querySelector('div#airportData').children;
                for(var i=0; i<aptTables.length; i++) 
                    if(aptTables[i].id==label) aptTables[i].className = "revealed";

                break;
            }
        }
    } // canvas.onclick
} // createClickHandler

/* --------------------------------------------------------------
   handleMapClick (icao1, icao2)
   --------------------------------------------------------------*/
function handleMapClick(icao1, icao2) {
    var apt1 = document.getElementById("calcApt1");
    var apt2 = document.getElementById("calcApt2");
    apt1.innerHTML = icao1;  // was apt2.innerHTML
    apt2.innerHTML = icao2;
    var apt1Ele = document.getElementById("calcApt1Ele");
    var apt1Lat = document.getElementById("calcApt1Lat");
    var apt1Lon = document.getElementById("calcApt1Lon");
    var apt2Ele = document.getElementById("calcApt2Ele");
    var apt2Lat = document.getElementById("calcApt2Lat");
    var apt2Lon = document.getElementById("calcApt2Lon");
    apt1Ele.innerHTML = apts[icao1].loc.elev+"'";
    apt1Lat.innerHTML = Dms.toLat(apts[icao1].loc.p.lat,"d");
    apt1Lon.innerHTML = Dms.toLon(apts[icao1].loc.p.lon,"d");
    apt2Ele.innerHTML = apts[icao2].loc.elev+"'";
    apt2Lat.innerHTML = Dms.toLat(apts[icao2].loc.p.lat,"d");
    apt2Lon.innerHTML = Dms.toLon(apts[icao2].loc.p.lon,"d");
    var gc1 = document.getElementById("calcGC1");
    var gc2 = document.getElementById("calcGC2");
    var rhumb = document.getElementById("calcRhumb");
    var gcBear1 = apts[icao1].loc.p.bearingTo(apts[icao2].loc.p);
    var gcBear2 = apts[icao1].loc.p.finalBearingTo(apts[icao2].loc.p);
    var rhumbBear = apts[icao1].loc.p.rhumbBearingTo(apts[icao2].loc.p);
    gc1.innerHTML = Dms.toBrng(gcBear1, "d",0);
    gc2.innerHTML = Dms.toBrng(gcBear2, "d",0);
    rhumb.innerHTML = Dms.toBrng(rhumbBear, "d",0);
    var gcDisNm = document.getElementById("calcGCDis");
    var rhDisNm = document.getElementById("calcRhumbDis");
    var gcDis = apts[icao1].loc.p.distanceTo(apts[icao2].loc.p,radiusM)/1852;
    var rhDis = apts[icao1].loc.p.rhumbDistanceTo(apts[icao2].loc.p,radiusM)/1852;
    gcDisNm.innerHTML = gcDis.toFixed(0)+" Nm";
    rhDisNm.innerHTML = rhDis.toFixed(0)+" Nm";

    redrawMap(icao1, icao2);
} // handleMapClick


